/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
$begin dbesks$$ 
$comment  The omhelp commands are documented in
	http://www.seanet.com/~bradbell/omhelp/
$$
$spell
	cmath
	exp
	std
	atan
	sqrt
	xnu
	bk
	nin
	Bessel
	Abramowitz
	Stegun
	dbesks
	cpp
	hpp
	bool
	doublereal
	adolc
	cppad
	fadbad
$$

$section dbesks$$

$index dbesks$$

$head Prototype$$
$include p.omh/dbesks.p.omh$$

$children%
	f.omh/dbesks.f.omh
%$$
$head Fortran Source$$
$cref dbesks.f$$


$head Notation$$
We use $latex K_{\nu} (x)$$ to denote the modified Bessel function
of the second kind; i.e., it is the solution of the ODE
$latex \[
x^2 y^{(2)} (x) + x y^{(1)} (x) - ( x^2 + \nu^2 ) = 0
\] $$
that goes to zero as $latex x \rightarrow \infty$$.

$head Description$$
The routine $codei dbesks$$ computes the values
$latex K_{\nu + i} (x)$$ where
$latex x > 0$$,
$latex -1 < \nu < 1$$,
$latex i = 0 , 1 , \ldots , nin-1$$  (if $latex nin > 0$$),
$latex i = 0 , -1 , \ldots , nin+1$$  (if $latex nin < 0$$).

$head Special Case$$
By Formula 10.2.17 of Abramowitz and Stegun 
$latex \[
	K_{3/2} ( x ) = \sqrt{  \pi / ( 2 x ) } ( 1 + 1 / x ) \exp( - x )
\] $$


$codep */

# include <cmath>
# include <f2cad/dbesks.hpp>

test_result dbesks(void)
{	bool ok = true;

	// independent variable
	integer    n = 1;          // number of independent variables
	doublereal x[1];           // vector of independent variables
	x[0]         = .75;        // value of independent variable
	f2cad::Independent(n, x);  // declare x as independent variable vector

	// call the routine
	doublereal xnu = .5;
	doublereal bk[2];
	integer    nin = 2;  
	f2cad::dbesks_(&xnu, x+0, &nin, bk);

	// dependent variables 
	integer     m = 1;          // number of dependent variables
	doublereal  f[1];           // vector of dependent variables
	f[0] = bk[1];               // value f[0] = K_(nu+1) (x[0])
	f2cad::Dependent(m, f);     // declare f as dependent variable vector

	// check function value
	double x0    = f2cad::Value( x[0] );
	double f0    = f2cad::Value( f[0] );
	double pi    = 4. * std::atan(1.);
	double term  = 1. / sqrt( 2. * x0 / pi );
	double check = term * (1. + 1. / x0 ) * exp(-x0);
	ok          &= f2cad::near_equal(f0, check, 1e-10, 1e-10);

	// Evaluate the derivative of f[0] w.r.t x[0]
	double p = f2cad::Partial<doublereal>(0, 0);
	check    = - check;
	check   += - term * exp(-x0) / (x0 * x0);
	check   += - (1. + 1. / x0) * exp(-x0) * term * term * term / pi;
	ok      &= f2cad::near_equal(p, check, 1e-10, 1e-10);

	if( ok ) 
		return test_pass;
	return test_fail;
}
/* $$
$end
*/
