/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
$begin d1mach$$
$spell
	doublereal
	cmlib
	cpp
	nerr
	const
$$

$section Implementation of Cmlib d1mach Routine in Cpp$$

$index d1mach, C++$$
$index C++, d1mach$$

$head Syntax$$ 
$icode%d% = d1mach_(%i%)%$$

$head Purpose$$
This routine uses the C++ standard $code numeric_limits$$ to
implement the specifications required by the cmlib Fortran routine 
$code d1mach$$.

$head i$$
The argument $icode i$$ has prototype
$codei%
	integer* %i%
%$$
It specifies which machine constant to evaluate.

$head d$$
The return value $icode d$$ has prototype
$code%
	doublereal %d%
%$$
and is the value of the requested machine constant.

$head Mapping$$
The Fortran definition for the machine constants below was
extracted from the comments in the Fortran routine $code d1mach$$.
The C++ definitions were extracted from the $code numeric_limits$$
section of the C++ standard. 

$table
$codei i$$ $cnext $codei d$$
$rnext
1          $cnext B**(EMIN-1), the smallest positive magnitude.
$rnext
2          $cnext B**EMAX*(1 - B**(-T)), the largest magnitude.
$rnext
3          $cnext B**(-T), the smallest relative spacing.
$rnext
4          $cnext  B**(1-T), the largest relative spacing.
$rnext
5          $cnext  LOG10(B).
$tend

$end
*/

# include <cstdio>
# include <cassert>
# include <cstdlib>

# include <limits>
# include <cassert>
# include <cmath>

# include <f2cad/f2cad.hpp>

// we should remove the definition of the macros min and max from f2cad.hpp
# undef min
# undef max

namespace f2cad {
	doublereal d1mach_(integer *i)
	{	static int radix       = std::numeric_limits<double>::radix;
		static double d1mach_5 = std::log( double(radix) ) / std::log(10.);
	
		double return_value;
		switch( *i )
		{
			case 1:
			// d1mach(1) = B**(EMIN-1), the smallest positive magnitude.
			return_value = std::numeric_limits<double>::min();
			break;
	
			case 2:
			// d1mach(2) = B**EMAX*(1 - B**(-T)), the largest magnitude.
			return_value = std::numeric_limits<double>::max();
			break;
	
			case 3:
			// d1mach(3) = B**(-T), the smallest relative spacing.
			return_value = std::numeric_limits<double>::epsilon();
			break;
	
			case 4:
			// d1mach(4) = B**(1-T), the largest relative spacing.
			return_value = double(radix) * 
				std::numeric_limits<double>::epsilon();
			break;
	
			case 5:
			// d1mach(5) = LOG10(B)
			return_value = d1mach_5;
			break;
	
			default:
			assert(false);
			return_value = 0.;
		} 	
		return doublereal( return_value );
	}
}
